@@ -18,24 +18,28 @@
 #endif
 
 #include "gettext_defs.h"
-#include "inotify_monitor.h"
+#include "inotify_monitor.hpp"
 #include <limits.h>
+#ifdef __sun
+#  define NAME_MAX         255    /* # chars in a file name */
+#endif
 #include <unistd.h>
 #include <stdio.h>
 #include <sstream>
 #include <ctime>
 #include <cmath>
 #include <sys/select.h>
-#include "libfswatch_exception.h"
+#include "libfswatch_exception.hpp"
 #include "../c/libfswatch_log.h"
-#include "libfswatch_map.h"
-#include "libfswatch_set.h"
-#include "path_utils.h"
+#include "libfswatch_map.hpp"
+#include "libfswatch_set.hpp"
+#include "path_utils.hpp"
 
 using namespace std;
 
 namespace fsw
 {
+
   struct inotify_monitor_impl
   {
     int inotify_monitor_handle = -1;
@@ -75,11 +79,11 @@ namespace fsw
     monitor(paths_to_monitor, callback, context),
     impl(new inotify_monitor_impl())
   {
-    impl->inotify_monitor_handle = ::inotify_init();
+    impl->inotify_monitor_handle = inotify_init();
 
     if (impl->inotify_monitor_handle == -1)
     {
-      ::perror("inotify_init");
+      perror("inotify_init");
       throw libfsw_exception(_("Cannot initialize inotify."));
     }
   }
@@ -91,18 +95,18 @@ namespace fsw
     {
       ostringstream log;
       log << _("Removing: ") << inotify_desc_pair << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
 
-      if (::inotify_rm_watch(impl->inotify_monitor_handle, inotify_desc_pair))
+      if (inotify_rm_watch(impl->inotify_monitor_handle, inotify_desc_pair))
       {
-        ::perror("inotify_rm_watch");
+        perror("inotify_rm_watch");
       }
     }
 
     // close inotify
     if (impl->inotify_monitor_handle > 0)
     {
-      ::close(impl->inotify_monitor_handle);
+      close(impl->inotify_monitor_handle);
     }
 
     delete impl;
@@ -112,13 +116,13 @@ namespace fsw
                                   const struct stat &fd_stat)
   {
     // TODO: Consider optionally adding the IN_EXCL_UNLINK flag.
-    int inotify_desc = ::inotify_add_watch(impl->inotify_monitor_handle,
-                                           path.c_str(),
-                                           IN_ALL_EVENTS);
+    int inotify_desc = inotify_add_watch(impl->inotify_monitor_handle,
+                                         path.c_str(),
+                                         IN_ALL_EVENTS);
 
     if (inotify_desc == -1)
     {
-      ::perror("inotify_add_watch");
+      perror("inotify_add_watch");
     }
     else
     {
@@ -128,7 +132,7 @@ namespace fsw
 
       ostringstream log;
       log << _("Added: ") << path << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
     }
 
     return (inotify_desc != -1);
@@ -137,30 +141,36 @@ namespace fsw
   void inotify_monitor::scan(const string &path, const bool accept_non_dirs)
   {
     struct stat fd_stat;
-    if (!stat_path(path, fd_stat)) return;
+    if (!lstat_path(path, fd_stat)) return;
+
+    if (follow_symlinks && S_ISLNK(fd_stat.st_mode))
+    {
+      string link_path;
+      if (read_link_path(path, link_path))
+        scan(link_path, accept_non_dirs);
+
+      return;
+    }
+
+    bool is_dir = S_ISDIR(fd_stat.st_mode);
 
     /*
      * When watching a directory the inotify API will return change events of
      * first-level children.  Therefore, we do not need to manually add a watch
      * for a child unless it is a directory.  By default, accept_non_dirs is
      * true to allow watching a file when first invoked on a node.
+     *
+     * For the same reason, the directory_only flag is ignored and treated as if
+     * it were always set to true.
      */
-    if (!accept_non_dirs && !S_ISDIR(fd_stat.st_mode)) return;
-    else if (follow_symlinks && S_ISLNK(fd_stat.st_mode))
-    {
-      string link_path;
-      if (read_link_path(path, link_path))
-        scan(link_path/*, fn */, accept_non_dirs);
+    if (!accept_non_dirs && !is_dir) return;
 
-      return;
-    }
+    if (!is_dir && directory_only && !accept_non_dirs) return;
+    if (!is_dir && !accept_path(path)) return;
+    if (!add_watch(path, fd_stat)) return;
+    if (!recursive || !is_dir) return;
 
-    if (!S_ISDIR(fd_stat.st_mode) && !accept_path(path)) return;
-    if (!add_watch(path, fd_stat /*, fn */)) return;
-    if (!recursive || !S_ISDIR(fd_stat.st_mode)) return;
-
-    vector<string> children;
-    get_directory_children(path, children);
+    vector<string> children = get_directory_children(path);
 
     for (const string &child : children)
     {
@@ -169,7 +179,7 @@ namespace fsw
       /*
        * Scan children but only watch directories.
        */
-      scan(path + "/" + child /*, fn */, false);
+      scan(path + "/" + child, false);
     }
   }
 
@@ -241,7 +251,7 @@ namespace fsw
     {
       ostringstream log;
       log << _("Generic event: ") << event->wd << "::" << filename_stream.str() << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
     }
 
     /*
@@ -252,7 +262,7 @@ namespace fsw
     {
       ostringstream log;
       log << "IN_IGNORED: " << event->wd << "::" << filename_stream.str() << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
 
       impl->descriptors_to_remove.insert(event->wd);
     }
@@ -265,15 +275,15 @@ namespace fsw
      *
      * Beware that a race condition exists which may result in events go
      * unnoticed when a watched file x is removed and a new file named x is
-     * created thereafter.  In this case, fswatch could be blocked on ::read
-     * and it would not have any chance to create a new watch descriptor for x
-     * until an event is received and ::read unblocks.
+     * created thereafter.  In this case, fswatch could be blocked on read and
+     * it would not have any chance to create a new watch descriptor for x until
+     *  an event is received and read unblocks.
      */
     if (event->mask & IN_MOVE_SELF)
     {
       ostringstream log;
       log << "IN_MOVE_SELF: " << event->wd << "::" << filename_stream.str() << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
 
       impl->watches_to_remove.insert(event->wd);
       impl->descriptors_to_remove.insert(event->wd);
@@ -292,7 +302,7 @@ namespace fsw
     {
       ostringstream log;
       log << "IN_DELETE_SELF: " << event->wd << "::" << filename_stream.str() << "\n";
-      libfsw_log(log.str().c_str());
+      FSW_ELOG(log.str().c_str());
 
       impl->descriptors_to_remove.insert(event->wd);
     }
@@ -302,7 +312,7 @@ namespace fsw
   {
     if (event->mask & IN_Q_OVERFLOW)
     {
-      throw libfsw_exception(_("Event queue overflowed."));
+      notify_overflow(impl->wd_to_path[event->wd]);
     }
 
     preprocess_dir_event(event);
@@ -325,15 +335,15 @@ namespace fsw
 
     while (wtd != impl->watches_to_remove.end())
     {
-      if (::inotify_rm_watch(impl->inotify_monitor_handle, *wtd) != 0)
+      if (inotify_rm_watch(impl->inotify_monitor_handle, *wtd) != 0)
       {
-        ::perror("inotify_rm_watch");
+        perror("inotify_rm_watch");
       }
       else
       {
         ostringstream log;
         log << _("Removed: ") << *wtd << "\n";
-        libfsw_log(log.str().c_str());
+        FSW_ELOG(log.str().c_str());
       }
 
       impl->watches_to_remove.erase(wtd++);
@@ -366,7 +376,7 @@ namespace fsw
       // If no files can be watched, sleep and repeat the loop.
       if (!impl->watched_descriptors.size())
       {
-        ::sleep(latency);
+        sleep(latency);
         continue;
       }
 
@@ -379,15 +389,15 @@ namespace fsw
       FD_ZERO(&set);
       FD_SET(impl->inotify_monitor_handle, &set);
       double sec;
-      double frac = ::modf(this->latency, &sec);
+      double frac = modf(this->latency, &sec);
       timeout.tv_sec = sec;
       timeout.tv_usec = 1000 * 1000 * frac;
 
-      int rv = ::select(impl->inotify_monitor_handle + 1,
-                        &set,
-                        nullptr,
-                        nullptr,
-                        &timeout);
+      int rv = select(impl->inotify_monitor_handle + 1,
+                      &set,
+                      nullptr,
+                      nullptr,
+                      &timeout);
 
       if (rv == -1)
       {
@@ -400,25 +410,25 @@ namespace fsw
         continue;
       }
 
-      ssize_t record_num = ::read(impl->inotify_monitor_handle,
-                                  buffer,
-                                  BUFFER_SIZE);
+      ssize_t record_num = read(impl->inotify_monitor_handle,
+                                buffer,
+                                BUFFER_SIZE);
 
       {
         ostringstream log;
         log << _("Number of records: ") << record_num << "\n";
-        libfsw_log(log.str().c_str());
+        FSW_ELOG(log.str().c_str());
       }
 
       if (!record_num)
       {
-        throw libfsw_exception(_("::read() on inotify descriptor read 0 records."));
+        throw libfsw_exception(_("read() on inotify descriptor read 0 records."));
       }
 
       if (record_num == -1)
       {
-        ::perror("read()");
-        throw libfsw_exception(_("::read() on inotify descriptor returned -1."));
+        perror("read()");
+        throw libfsw_exception(_("read() on inotify descriptor returned -1."));
       }
 
       time(&impl->curr_time);
@@ -439,7 +449,7 @@ namespace fsw
         impl->events.clear();
       }
 
-      ::sleep(latency);
+      sleep(latency);
     }
   }
 }
