@@ -20,12 +20,12 @@
 #ifdef HAVE_SYS_EVENT_H
 
 #  include "gettext_defs.h"
-#  include "kqueue_monitor.h"
-#  include "libfswatch_map.h"
-#  include "libfswatch_set.h"
-#  include "libfswatch_exception.h"
+#  include "kqueue_monitor.hpp"
+#  include "libfswatch_map.hpp"
+#  include "libfswatch_set.hpp"
+#  include "libfswatch_exception.hpp"
 #  include "../c/libfswatch_log.h"
-#  include "path_utils.h"
+#  include "path_utils.hpp"
 #  include <iostream>
 #  include <sys/types.h>
 #  include <ctime>
@@ -38,13 +38,41 @@ using namespace std;
 
 namespace fsw
 {
+
   struct kqueue_monitor_load
   {
-    fsw_hash_map<std::string, int> descriptors_by_file_name;
-    fsw_hash_map<int, std::string> file_names_by_descriptor;
+    fsw_hash_map<string, int> descriptors_by_file_name;
+    fsw_hash_map<int, string> file_names_by_descriptor;
+    fsw_hash_map<int, mode_t> file_modes;
     fsw_hash_set<int> descriptors_to_remove;
     fsw_hash_set<int> descriptors_to_rescan;
-    fsw_hash_map<int, mode_t> file_modes;
+
+    void add_watch(int fd, const string & path, const struct stat &fd_stat)
+    {
+      descriptors_by_file_name[path] = fd;
+      file_names_by_descriptor[fd] = path;
+      file_modes[fd] = fd_stat.st_mode;
+    }
+
+    void remove_watch(int fd)
+    {
+      string name = file_names_by_descriptor[fd];
+      file_names_by_descriptor.erase(fd);
+      descriptors_by_file_name.erase(name);
+      file_modes.erase(fd);
+
+      close(fd);
+    }
+
+    void remove_watch(const string & path)
+    {
+      int fd = descriptors_by_file_name[path];
+      descriptors_by_file_name.erase(path);
+      file_names_by_descriptor.erase(fd);
+      file_modes.erase(fd);
+
+      close(fd);
+    }
   };
 
   typedef struct KqueueFlagType
@@ -80,7 +108,7 @@ namespace fsw
 
   kqueue_monitor::~kqueue_monitor()
   {
-    if (kq != -1) ::close(kq);
+    if (kq != -1) close(kq);
     delete load;
   }
 
@@ -129,37 +157,33 @@ namespace fsw
     int o_flags = 0;
 #  ifdef O_SYMLINK
     o_flags |= O_SYMLINK;
-#  elif defined(O_NOFOLLOW)
-    o_flags |= O_NOFOLLOW;
 #  endif
 #  ifdef O_EVTONLY
+    // The descriptor is requested for event notifications only.
     o_flags |= O_EVTONLY;
 #  else
     o_flags |= O_RDONLY;
 #  endif
 
-    int fd = ::open(path.c_str(), o_flags);
+    int fd = open(path.c_str(), o_flags);
 
     if (fd == -1)
     {
-      string err = string(_("Cannot open ")) + path;
-      libfsw_perror(err.c_str());
+      fsw_logf_perror(_("Cannot open %s"), path.c_str());
 
       return false;
     }
 
     // if the descriptor could be opened, track it
-    load->descriptors_by_file_name[path] = fd;
-    load->file_names_by_descriptor[fd] = path;
-    load->file_modes[fd] = fd_stat.st_mode;
+    load->add_watch(fd, path, fd_stat);
 
     return true;
   }
 
-  bool kqueue_monitor::scan(const string &path)
+  bool kqueue_monitor::scan(const string &path, bool is_root_path)
   {
     struct stat fd_stat;
-    if (!stat_path(path, fd_stat)) return false;
+    if (!lstat_path(path, fd_stat)) return false;
 
     if (follow_symlinks && S_ISLNK(fd_stat.st_mode))
     {
@@ -170,50 +194,33 @@ namespace fsw
       return false;
     }
 
-    if (!S_ISDIR(fd_stat.st_mode) && !accept_path(path)) return true;
+    bool is_dir = S_ISDIR(fd_stat.st_mode);
+
+    if (!is_dir && !is_root_path && directory_only) return true;
+    if (!is_dir && !accept_path(path)) return true;
     if (!add_watch(path, fd_stat)) return false;
     if (!recursive) return true;
-    if (!S_ISDIR(fd_stat.st_mode)) return true;
+    if (!is_dir) return true;
 
-    vector<string> children;
-    get_directory_children(path, children);
+    vector<string> children = get_directory_children(path);
 
-    for (string &child : children)
+    for (string & child : children)
     {
       if (child.compare(".") == 0 || child.compare("..") == 0) continue;
 
-      scan(path + "/" + child);
+      scan(path + "/" + child, false);
     }
 
     return true;
   }
 
-  void kqueue_monitor::remove_watch(int fd)
-  {
-    string name = load->file_names_by_descriptor[fd];
-    load->file_names_by_descriptor.erase(fd);
-    load->descriptors_by_file_name.erase(name);
-    load->file_modes.erase(fd);
-    ::close(fd);
-  }
-
-  void kqueue_monitor::remove_watch(const string &path)
-  {
-    int fd = load->descriptors_by_file_name[path];
-    load->descriptors_by_file_name.erase(path);
-    load->file_names_by_descriptor.erase(fd);
-    load->file_modes.erase(fd);
-    ::close(fd);
-  }
-
   void kqueue_monitor::remove_deleted()
   {
     auto fd = load->descriptors_to_remove.begin();
 
     while (fd != load->descriptors_to_remove.end())
     {
-      remove_watch(*fd);
-
+      load->remove_watch(*fd);
       load->descriptors_to_remove.erase(fd++);
     }
   }
@@ -236,7 +243,8 @@ namespace fsw
       // If the descriptor which has vanished is a directory, we don't bother
       // EV_DELETEing all its children the event from kqueue for the same
       // reason.
-      remove_watch(fd_path);
+      load->remove_watch(fd_path);
+
       scan(fd_path);
 
       load->descriptors_to_rescan.erase(fd++);
@@ -251,8 +259,7 @@ namespace fsw
 
       if (!scan(path))
       {
-        string err = _("Notice: ") + path + _(" cannot be found. Will retry later.\n");
-        libfsw_log(err.c_str());
+        FSW_ELOGF(_("%s cannot be found. Will retry later.\n"), path.c_str());
       }
     }
   }
@@ -261,11 +268,11 @@ namespace fsw
   {
     if (kq != -1) throw libfsw_exception(_("kqueue already running."));
 
-    kq = ::kqueue();
+    kq = kqueue();
 
     if (kq == -1)
     {
-      perror("::kqueue()");
+      perror("kqueue()");
       throw libfsw_exception(_("kqueue failed."));
     }
   }
@@ -275,24 +282,24 @@ namespace fsw
   {
     struct timespec ts = create_timespec_from_latency(latency);
 
-    int event_num = ::kevent(kq,
-                             &changes[0],
-                             changes.size(),
-                             &event_list[0],
-                             event_list.size(),
-                             &ts);
+    int event_num = kevent(kq,
+                           &changes[0],
+                           changes.size(),
+                           &event_list[0],
+                           event_list.size(),
+                           &ts);
 
     if (event_num == -1)
     {
-      ::perror("::kevent");
-      throw libfsw_exception(_("::kevent returned -1, invalid event number."));
+      perror("kevent");
+      throw libfsw_exception(_("kevent returned -1, invalid event number."));
     }
 
     return event_num;
   }
 
-  void kqueue_monitor::process_events(const vector<struct ::kevent> &changes,
-                                      const vector<struct ::kevent> &event_list,
+  void kqueue_monitor::process_events(const vector<struct kevent> &changes,
+                                      const vector<struct kevent> &event_list,
                                       int event_num)
   {
     time_t curr_time;
@@ -301,11 +308,11 @@ namespace fsw
 
     for (auto i = 0; i < event_num; ++i)
     {
-      struct ::kevent e = event_list[i];
+      struct kevent e = event_list[i];
 
       if (e.flags & EV_ERROR)
       {
-        ::perror(_("Event with EV_ERROR"));
+        perror(_("Event with EV_ERROR"));
         continue;
       }
 
@@ -362,7 +369,7 @@ namespace fsw
       // scan the root paths to check whether someone is missing
       scan_root_paths();
 
-      vector<struct ::kevent> changes;
+      vector<struct kevent> changes;
       vector<struct kevent> event_list;
 
       for (const pair<int, string> &fd_path : load->file_names_by_descriptor)
@@ -387,7 +394,7 @@ namespace fsw
        */
       if (!changes.size())
       {
-        ::sleep(latency);
+        sleep(latency);
         continue;
       }
 
