@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#include "libfswatch_exception.h"
+#include "libfswatch_exception.hpp"
 #include "gettext_defs.h"
 
 using namespace std;
