@@ -14,10 +14,11 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include "gettext_defs.h"
-#include "path_utils.h"
+#include "path_utils.hpp"
 #include "c/libfswatch_log.h"
 #include <dirent.h>
 #include <cstdlib>
+#include <cstdio>
 #include <errno.h>
 #include <iostream>
 
@@ -25,23 +26,24 @@ using namespace std;
 
 namespace fsw
 {
-  void get_directory_children(const string &path, vector<string> &children)
+
+  vector<string> get_directory_children(const string &path)
   {
-    DIR *dir = ::opendir(path.c_str());
+    vector<string> children;
+    DIR *dir = opendir(path.c_str());
 
     if (!dir)
     {
       if (errno == EMFILE || errno == ENFILE)
       {
         perror("opendir");
-        // ::exit(FSW_EXIT_ENFILE);
       }
       else
       {
-        libfsw_perror("opendir");
+        fsw_log_perror("opendir");
       }
 
-      return;
+      return children;
     }
 
     while (struct dirent * ent = readdir(dir))
@@ -49,26 +51,39 @@ namespace fsw
       children.push_back(ent->d_name);
     }
 
-    ::closedir(dir);
+    closedir(dir);
+
+    return children;
   }
 
   bool read_link_path(const string &path, string &link_path)
   {
-    char *real_path = ::realpath(path.c_str(), nullptr);
+    char *real_path = realpath(path.c_str(), nullptr);
     link_path = (real_path ? real_path : path);
 
     bool ret = (real_path != nullptr);
-    ::free(real_path);
+    free(real_path);
 
     return ret;
   }
 
   bool stat_path(const string &path, struct stat &fd_stat)
   {
-    if (::lstat(path.c_str(), &fd_stat) != 0)
+    if (stat(path.c_str(), &fd_stat) != 0)
     {
-      string err = string(_("Cannot stat() ")) + path;
-      libfsw_perror(err.c_str());
+      fsw_logf_perror(_("Cannot stat %s"), path.c_str());
+
+      return false;
+    }
+
+    return true;
+  }
+
+  bool lstat_path(const string &path, struct stat &fd_stat)
+  {
+    if (lstat(path.c_str(), &fd_stat) != 0)
+    {
+      fsw_logf_perror(_("Cannot lstat %s"), path.c_str());
 
       return false;
     }
