@@ -18,14 +18,14 @@
 #endif
 
 #include "gettext_defs.h"
-#include "poll_monitor.h"
+#include "poll_monitor.hpp"
 #include <unistd.h>
 #include <cstdlib>
 #include <fcntl.h>
 #include <iostream>
 #include "c/libfswatch_log.h"
-#include "path_utils.h"
-#include "libfswatch_map.h"
+#include "path_utils.hpp"
+#include "libfswatch_map.hpp"
 
 using namespace std;
 
@@ -41,7 +41,7 @@ namespace fsw
 {
   typedef struct poll_monitor::poll_monitor_data
   {
-    fsw_hash_map<std::string, poll_monitor::watched_file_info> tracked_files;
+    fsw_hash_map<string, poll_monitor::watched_file_info> tracked_files;
   }
   poll_monitor_data;
 
@@ -125,7 +125,7 @@ namespace fsw
   void poll_monitor::scan(const string &path, poll_monitor_scan_callback fn)
   {
     struct stat fd_stat;
-    if (!stat_path(path, fd_stat)) return;
+    if (!lstat_path(path, fd_stat)) return;
 
     if (follow_symlinks && S_ISLNK(fd_stat.st_mode))
     {
@@ -141,8 +141,7 @@ namespace fsw
     if (!recursive) return;
     if (!S_ISDIR(fd_stat.st_mode)) return;
 
-    vector<string> children;
-    get_directory_children(path, children);
+    vector<string> children = get_directory_children(path);
 
     for (string &child : children)
     {
@@ -159,7 +158,7 @@ namespace fsw
 
     for (auto &removed : previous_data->tracked_files)
     {
-        events.push_back({removed.first, curr_time, flags});
+      events.push_back({removed.first, curr_time, flags});
     }
   }
 
@@ -199,11 +198,9 @@ namespace fsw
 
     while (true)
     {
-#ifdef DEBUG
-      libfsw_log(_("Done scanning.\n"));
-#endif
+      FSW_ELOG(_("Done scanning.\n"));
 
-      ::sleep(latency < MIN_POLL_LATENCY ? MIN_POLL_LATENCY : latency);
+      sleep(latency < MIN_POLL_LATENCY ? MIN_POLL_LATENCY : latency);
 
       time(&curr_time);
 
