@@ -34,7 +34,9 @@ extern "C"
     fsevents_monitor_type,
     kqueue_monitor_type,
     inotify_monitor_type,
-    poll_monitor_type
+    windows_monitor_type,
+    poll_monitor_type,
+    fen_monitor_type
   };
 
 #  ifdef __cplusplus
