@@ -24,14 +24,16 @@
 #  include <atomic>
 #endif
 #include <ctime>
-#include <stdlib.h>
+#include <cstdlib>
 #include <cstring>
 #include <memory>
+#include <vector>
+#include <map>
 #include "libfswatch.h"
-#include "../c++/libfswatch_map.h"
-#include "../c++/filter.h"
-#include "../c++/monitor.h"
-#include "../c++/libfswatch_exception.h"
+#include "../c++/libfswatch_map.hpp"
+#include "../c++/filter.hpp"
+#include "../c++/monitor.hpp"
+#include "../c++/libfswatch_exception.hpp"
 
 using namespace std;
 using namespace fsw;
@@ -44,10 +46,13 @@ typedef struct FSW_SESSION
   fsw::monitor * monitor;
   FSW_CEVENT_CALLBACK callback;
   double latency;
+  bool allow_overflow;
   bool recursive;
+  bool directory_only;
   bool follow_symlinks;
   vector<monitor_filter> filters;
   vector<fsw_event_type_filter> event_type_filters;
+  map<string, string> properties;
   void * data;
 #ifdef HAVE_CXX_MUTEX
   atomic<bool> running;
@@ -55,6 +60,7 @@ typedef struct FSW_SESSION
 } FSW_SESSION;
 
 static bool srand_initialized = false;
+static bool fsw_libfswatch_verbose = false;
 
 #ifdef HAVE_CXX_UNIQUE_PTR
 static fsw_hash_map<FSW_HANDLE, unique_ptr<FSW_SESSION>> sessions;
@@ -69,14 +75,18 @@ static fsw_hash_map<FSW_HANDLE, unique_ptr<mutex>> session_mutexes;
 static fsw_hash_map<FSW_HANDLE, mutex *> session_mutexes;
 #  endif
 static std::mutex session_mutex;
+
+#  define SESSION_GUARD \
+    std::lock_guard<std::mutex> session_lock(session_mutex);
+#else
+#  define SESSION_GUARD
 #endif
 
 static FSW_THREAD_LOCAL FSW_STATUS last_error;
 
-// Default library callback.
+// Forward declarations.
 static FSW_EVENT_CALLBACK libfsw_cpp_callback_proxy;
 static FSW_SESSION * get_session(const FSW_HANDLE handle);
-
 static int create_monitor(FSW_HANDLE handle, const fsw_monitor_type type);
 static FSW_STATUS fsw_set_last_error(const int error);
 
@@ -110,7 +120,7 @@ void libfsw_cpp_callback_proxy(const std::vector<event> & events,
 
   const fsw_callback_context * context = static_cast<fsw_callback_context *> (context_ptr);
 
-  fsw_cevent * const cevents = static_cast<fsw_cevent *> (::malloc(sizeof (fsw_cevent) * events.size()));
+  fsw_cevent * const cevents = static_cast<fsw_cevent *> (malloc(sizeof (fsw_cevent) * events.size()));
 
   if (cevents == nullptr)
     throw int(FSW_ERR_MEMORY);
@@ -124,12 +134,11 @@ void libfsw_cpp_callback_proxy(const std::vector<event> & events,
     const string path = evt.get_path();
 
     // Copy std::string into char * buffer and null-terminate it.
-    cevt->path = static_cast<char *> (::malloc(sizeof (char *) * (path.length() + 1)));
+    cevt->path = static_cast<char *> (malloc(sizeof (char *) * (path.length() + 1)));
     if (!cevt->path) throw int(FSW_ERR_MEMORY);
 
-    ::strncpy(cevt->path, path.c_str(), path.length());
+    strncpy(cevt->path, path.c_str(), path.length());
     cevt->path[path.length()] = '\0';
-
     cevt->evt_time = evt.get_time();
 
     const vector<fsw_event_flag> flags = evt.get_flags();
@@ -138,7 +147,7 @@ void libfsw_cpp_callback_proxy(const std::vector<event> & events,
     if (!cevt->flags_num) cevt->flags = nullptr;
     else
     {
-      cevt->flags = static_cast<fsw_event_flag *> (::malloc(sizeof (fsw_event_flag) * cevt->flags_num));
+      cevt->flags = static_cast<fsw_event_flag *> (malloc(sizeof (fsw_event_flag) * cevt->flags_num));
       if (!cevt->flags) throw int(FSW_ERR_MEMORY);
     }
 
@@ -156,18 +165,16 @@ void libfsw_cpp_callback_proxy(const std::vector<event> & events,
   {
     fsw_cevent * cevt = &cevents[i];
 
-    if (cevt->flags) ::free(static_cast<void *> (cevt->flags));
-    ::free(static_cast<void *> (cevt->path));
+    if (cevt->flags) free(static_cast<void *> (cevt->flags));
+    free(static_cast<void *> (cevt->path));
   }
 
-  ::free(static_cast<void *> (cevents));
+  free(static_cast<void *> (cevents));
 }
 
 FSW_HANDLE fsw_init_session(const fsw_monitor_type type)
 {
-#ifdef HAVE_CXX_MUTEX
-  std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+  SESSION_GUARD;
 
   if (!srand_initialized)
   {
@@ -233,7 +240,7 @@ int create_monitor(const FSW_HANDLE handle, const fsw_monitor_type type)
                                                                 context_ptr);
     session->monitor = current_monitor;
   }
-  catch (libfsw_exception ex)
+  catch (libfsw_exception & ex)
   {
     return fsw_set_last_error(int(ex));
   }
@@ -252,12 +259,31 @@ FSW_STATUS fsw_add_path(const FSW_HANDLE handle, const char * path)
 
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
+    FSW_SESSION * session = get_session(handle);
+    session->paths.push_back(path);
+  }
+  catch (int error)
+  {
+    return fsw_set_last_error(error);
+  }
+
+  return fsw_set_last_error(FSW_OK);
+}
+
+FSW_STATUS fsw_add_property(const FSW_HANDLE handle, const char * name, const char * value)
+{
+  if (!name || !value)
+    return fsw_set_last_error(FSW_ERR_INVALID_PROPERTY);
+
+  try
+  {
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
-    session->paths.push_back(path);
+    session->properties[name] = value;
   }
   catch (int error)
   {
@@ -274,9 +300,8 @@ FSW_STATUS fsw_set_callback(const FSW_HANDLE handle, const FSW_CEVENT_CALLBACK c
 
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->callback = callback;
@@ -290,6 +315,24 @@ FSW_STATUS fsw_set_callback(const FSW_HANDLE handle, const FSW_CEVENT_CALLBACK c
   return fsw_set_last_error(FSW_OK);
 }
 
+FSW_STATUS fsw_set_allow_overflow(const FSW_HANDLE handle, const bool allow_overflow)
+{
+  try
+  {
+    SESSION_GUARD;
+
+    FSW_SESSION * session = get_session(handle);
+
+    session->allow_overflow = allow_overflow;
+  }
+  catch (int error)
+  {
+    return fsw_set_last_error(error);
+  }
+
+  return fsw_set_last_error(FSW_OK);
+}
+
 FSW_STATUS fsw_set_latency(const FSW_HANDLE handle, const double latency)
 {
   if (latency < 0)
@@ -297,9 +340,8 @@ FSW_STATUS fsw_set_latency(const FSW_HANDLE handle, const double latency)
 
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->latency = latency;
@@ -316,9 +358,8 @@ FSW_STATUS fsw_set_recursive(const FSW_HANDLE handle, const bool recursive)
 {
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->recursive = recursive;
@@ -331,14 +372,31 @@ FSW_STATUS fsw_set_recursive(const FSW_HANDLE handle, const bool recursive)
   return fsw_set_last_error(FSW_OK);
 }
 
-FSW_STATUS fsw_set_follow_symlinks(const FSW_HANDLE handle,
-                            const bool follow_symlinks)
+FSW_STATUS fsw_set_directory_only(const FSW_HANDLE handle, const bool directory_only)
 {
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
+    FSW_SESSION * session = get_session(handle);
+
+    session->directory_only = directory_only;
+  }
+  catch (int error)
+  {
+    return fsw_set_last_error(error);
+  }
+
+  return fsw_set_last_error(FSW_OK);
+}
+
+FSW_STATUS fsw_set_follow_symlinks(const FSW_HANDLE handle,
+                                   const bool follow_symlinks)
+{
+  try
+  {
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->follow_symlinks = follow_symlinks;
@@ -356,9 +414,8 @@ FSW_STATUS fsw_add_event_type_filter(const FSW_HANDLE handle,
 {
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->event_type_filters.push_back(event_type);
@@ -372,13 +429,12 @@ FSW_STATUS fsw_add_event_type_filter(const FSW_HANDLE handle,
 }
 
 FSW_STATUS fsw_add_filter(const FSW_HANDLE handle,
-                   const fsw_cmonitor_filter filter)
+                          const fsw_cmonitor_filter filter)
 {
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
     session->filters.push_back({filter.text, filter.type, filter.case_sensitive, filter.extended});
@@ -444,11 +500,13 @@ FSW_STATUS fsw_start_monitor(const FSW_HANDLE handle)
     if (!session->monitor)
       create_monitor(handle, session->type);
 
+    session->monitor->set_allow_overflow(session->allow_overflow);
     session->monitor->set_filters(session->filters);
     session->monitor->set_event_type_filters(session->event_type_filters);
     session->monitor->set_follow_symlinks(session->follow_symlinks);
     if (session->latency) session->monitor->set_latency(session->latency);
     session->monitor->set_recursive(session->recursive);
+    session->monitor->set_directory_only(session->directory_only);
 
 #ifdef HAVE_CXX_MUTEX
     session->running.store(true, memory_order_release);
@@ -457,6 +515,10 @@ FSW_STATUS fsw_start_monitor(const FSW_HANDLE handle)
 
     session->monitor->start();
   }
+  catch (libfsw_exception & ex)
+  {
+    return fsw_set_last_error(int(ex));
+  }
   catch (int error)
   {
     return fsw_set_last_error(error);
@@ -471,9 +533,8 @@ FSW_STATUS fsw_destroy_session(const FSW_HANDLE handle)
 
   try
   {
-#ifdef HAVE_CXX_MUTEX
-    std::lock_guard<std::mutex> session_lock(session_mutex);
-#endif
+    SESSION_GUARD;
+
     FSW_SESSION * session = get_session(handle);
 
 #ifdef HAVE_CXX_MUTEX
@@ -538,5 +599,10 @@ FSW_STATUS fsw_last_error()
 
 bool fsw_is_verbose()
 {
-  return false;
+  return fsw_libfswatch_verbose;
+}
+
+void fsw_set_verbose(bool verbose)
+{
+  fsw_libfswatch_verbose = verbose;
 }
